//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-16
// Updated : 2006-01-17
// Licence : This source is under GNU LGPL licence
// File    : _vec2.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __vec2_inl__
#define __vec2_inl__

#include "./_vec2.h"
#include "./_vec3.h"
#include "./_vec4.h"

namespace glm
{
    //////////////////////////////////////////////////////////////
    // vec2 and ivec2 Constructors

    template <typename T> 
    inline _xvec2<T>::_xvec2() : 
        x(0),
        y(0)
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(T f) : 
        x(f),
        y(f)
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(T x, T y) :
        x(x),
        y(y)
    {}
/*
    template <typename T> 
    inline _xvec2<T>::_xvec2(const T* a) :
        x(a[0]),
        y(a[1])
    {}
*/
    //////////////////////////////////////////////////////////////
    // vec2 and ivec2 conversions

    template <typename T> 
    inline _xvec2<T>::_xvec2(const glm::_xvec3<T>& v) :
        x(v.x),
        y(v.y)
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(const glm::_xvec4<T>& v) :
        x(v.x),
        y(v.y)
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(const glm::_bvec2& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(const glm::_bvec3& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    template <typename T> 
    inline _xvec2<T>::_xvec2(const glm::_bvec4& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    template <typename T> 
    template <typename U> 
    inline _xvec2<T>::_xvec2(U x, U y) :
        x(T(x)),
        y(T(y))
    {}

    template <typename T> 
    template <typename U>
    inline _xvec2<T>::_xvec2(const _xvec2<U>& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    template <typename T> 
    template <typename U>
    inline _xvec2<T>::_xvec2(const _xvec3<U>& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    template <typename T> 
    template <typename U>
    inline _xvec2<T>::_xvec2(const _xvec4<U>& v) :
        x(T(v.x)),
        y(T(v.y))
    {}

    //////////////////////////////////////////////////////////////
    // vec2 and ivec2 accesses

    template <typename T> 
    inline T& _xvec2<T>::operator [] (int i)
    {
        return (&x)[i];
    }

    template <typename T> 
    inline T _xvec2<T>::operator [] (int i) const
    {
        return (&x)[i];
    }

    template <typename T> 
    inline _xvec2<T>::operator T* ()
    {
        return &x;
    }

    template <typename T> 
    inline _xvec2<T>::operator const T* () const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // vec2 and ivec2 operators

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    template <typename T> 
    inline _xvec2<T>& _xvec2<T>::operator=(const _xvec2<T>& x)
    {
        this->x = x.x;
        this->y = x.y;
        return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator+ (const T s) const
    {
	    return _xvec2<T>(
		    this->x + s,
		    this->y + s);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator+= (const T s)
    {
	    this->x += s;
	    this->y += s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator+ (const glm::_xvec2<T> & v) const
    {
	    return _xvec2<T>(
		    this->x + v.x,
		    this->y + v.y);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator+= (const glm::_xvec2<T> & v)
    {
	    this->x += v.x;
	    this->y += v.y;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator- (const T s) const
    {
	    return _xvec2<T>(
		    this->x - s,
		    this->y - s);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator-= (const T s)
    {
	    this->x -= s;
	    this->y -= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator- (const glm::_xvec2<T> & v) const
    {
	    return _xvec2<T>(
		    this->x - v.x,
		    this->y - v.y);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator-= (const glm::_xvec2<T> & v)
    {
	    this->x -= v.x;
	    this->y -= v.y;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator* (const T s) const
    {
	    return _xvec2<T>(
		    this->x * s,
		    this->y * s);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator*= (const T s)
    {
	    this->x *= s;
	    this->y *= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator* (const _xvec2<T> & v) const
    {
	    return _xvec2<T>(
		    this->x * v.x,
		    this->y * v.y);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator*= (const _xvec2<T> & v)
    {
	    this->x *= v.x;
	    this->y *= v.y;
	    return *this;
    }

    template <typename T> 
    inline _xvec2<T> _xvec2<T>::operator* (const _xmat2<T>& m) const
    {
        return _xvec2<T>(
            m[0][0] * this->x + m[0][1] * this->y,
            m[1][0] * this->x + m[1][1] * this->y);
    }

    template <typename T> 
    inline _xvec2<T>& _xvec2<T>::operator*= (const _xmat2<T>& m)
    {
        return *this = *this * m;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator/ (const T s) const
    {
	    return _xvec2<T>(
		    this->x / s,
		    this->y / s);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator/= (const T s)
    {
	    this->x /= s;
	    this->y /= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator/ (const _xvec2<T> & v) const
    {
	    return _xvec2<T>(
		    this->x / v.x,
		    this->y / v.y);
    }

    template <typename T> 
    inline glm::_xvec2<T>& _xvec2<T>::operator/= (const _xvec2<T> & v)
    {
	    this->x /= v.x;
	    this->y /= v.y;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator- () const
    {
	    return _xvec2<T>(
            -this->x, 
            -this->y);
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator-- ()
    {
	    this->x--;
	    this->y--;
	    return *this;
    }

    template <typename T> 
    inline const glm::_xvec2<T> _xvec2<T>::operator-- (int n) const 
    {
		return _xvec2<T>(
			this->x - T(1), 
			this->y - T(1));
    }

    template <typename T> 
    inline glm::_xvec2<T> _xvec2<T>::operator++ ()
    {
	    this->x++;
	    this->y++;
	    return *this;
    }

    template <typename T> 
    inline const glm::_xvec2<T> _xvec2<T>::operator++ (int n) const
    {
		return _xvec2<T>(
			this->x + T(1), 
			this->y + T(1));
    }

    //////////////////////////////////////////////////////////////
    // bvec2 constructors

    inline _bvec2::_bvec2() :
        x(false),
        y(false)
    {}

    inline _bvec2::_bvec2(bool _x, bool _y) : 
        x(_x),
        y(_y)
    {}

    inline _bvec2::_bvec2(bool b) : 
        x(b),
        y(b)
    {}

    inline _bvec2::_bvec2(const bool* a) :
        x(a[0]),
        y(a[1])
    {}

    //////////////////////////////////////////////////////////////
    // bvec2 conversions

    template <typename U> 
    inline _bvec2::_bvec2(U x, U y) :
        x(bool(x)),
        y(bool(y))
    {}

    inline _bvec2::_bvec2(const _bvec3& v) :
        x(v.x),
        y(v.y)
    {}

    inline _bvec2::_bvec2(const _bvec4& v) :
        x(v.x),
        y(v.y)
    {}

    template <typename U>
    inline _bvec2::_bvec2(const _xvec2<U>& v) :
        x(bool(v.x)),
        y(bool(v.y))
    {}

    template <typename U>
    inline _bvec2::_bvec2(const _xvec3<U>& v) :
        x(bool(v.x)),
        y(bool(v.y))
    {}

    template <typename U>
    inline _bvec2::_bvec2(const _xvec4<U>& v) :
        x(bool(v.x)),
        y(bool(v.y))
    {}

    //////////////////////////////////////////////////////////////
    // bvec2 accesses

    inline bool& _bvec2::operator[](int i)
    {
        return (&x)[i];
    }

    inline bool _bvec2::operator[](int i) const
    {
        return (&x)[i];
    }

    inline _bvec2::operator bool*()
    {
        return &x;
    }

    inline _bvec2::operator const bool*() const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // bvec2 operators

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    inline _bvec2& _bvec2::operator=(const _bvec2& x)
    {
        this->x = x.x;
        this->y = x.y;
        return *this;
    }

    inline _bvec2 _bvec2::operator--()
    {
	    x = !x;
	    y = !y;
	    return *this;
    }

    inline _bvec2 _bvec2::operator++()
    {
	    x = !x;
	    y = !y;
	    return *this;
    }

    inline const _bvec2 _bvec2::operator--(int n) const 
    {
        return _bvec2(
			!this->x, 
			!this->y);
    }

    inline const _bvec2 _bvec2::operator++(int n) const
    {
	    return _bvec2(
			!this->x, 
			!this->y);
    }

    inline _bvec2 _bvec2::operator!() const 
    {
	    return _bvec2(
			!this->x, 
			!this->y);
    }
} //namespace glm

#endif //__vec2_inl__
