//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-16
// Updated : 2006-01-17
// Licence : This source is under GNU LGPL licence
// File    : _vec3.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __vec3_inl__
#define __vec3_inl__

#include "./_vec2.h"
#include "./_vec3.h"
#include "./_vec4.h"

namespace glm
{
    //////////////////////////////////////////////////////////////
    // vec3 and ivec3 constructors

    template <typename T> 
    inline _xvec3<T>::_xvec3() :
        x(0),
        y(0),
        z(0)
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(T f) :
        x(f),
        y(f),
        z(f)
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const T _x, const T _y, const T _z) :
        x(_x),
        y(_y),
        z(_z)
    {}
/*
    template <typename T> 
    inline _xvec3<T>::_xvec3(const T* a) :
        x(a[0]),
        y(a[1]),
        z(a[2])
    {}
*/
    //////////////////////////////////////////////////////////////
    // vec3 and ivec3 conversions

    template <typename T>
    template <typename U> 
    inline _xvec3<T>::_xvec3(U x, U y, U z) :
        x(T(x)),
        y(T(y)),
        z(T(z))
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const _xvec2<T>& v1, T v2) :
        x(v1.x),
        y(v1.y),
        z(v2)
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(T v1, const _xvec2<T>& v2) :
        x(v1),
        y(v2.x),
        z(v2.y)
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const _xvec4<T>& v) :
        x(v.x),
        y(v.y),
        z(v.z)    
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const _bvec3& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z))
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const _bvec2& v1, bool v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2))
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(bool v1, const _bvec2& v2) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y))
    {}

    template <typename T> 
    inline _xvec3<T>::_xvec3(const _bvec4& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z))
    {}

    template <typename T>
    template <typename U>
    inline _xvec3<T>::_xvec3(const _xvec3<U>& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z))
    {}

    template <typename T>
    template <typename U>
    inline _xvec3<T>::_xvec3(const _xvec2<U>& v1, U v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2))
    {}

    template <typename T>
    template <typename U>
    inline _xvec3<T>::_xvec3(U v1, const _xvec2<U>& v2) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y))
    {}

    template <typename T>
    template <typename U>
    inline _xvec3<T>::_xvec3(const _xvec4<U>& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z))
    {}

    //////////////////////////////////////////////////////////////
    // vec3 and ivec3 accesses

    template <typename T> 
    inline T& _xvec3<T>::operator [] (int i)
    {
        return (&x)[i];
    }

    template <typename T> 
    inline T _xvec3<T>::operator [] (int i) const
    {
        return (&x)[i];
    }

    template <typename T> 
    inline _xvec3<T>::operator T* ()
    {
        return &x;
    }

    template <typename T> 
    inline _xvec3<T>::operator const T* () const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // vec3 and ivec3 operators

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    template <typename T> 
    inline _xvec3<T>& _xvec3<T>::operator=(const _xvec3<T>& x)
    {
        this->x = x.x;
        this->y = x.y;
        this->z = x.z;
        return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator+ (const T s) const
    {
	    return glm::_xvec3<T>(
		    this->x + s,
		    this->y + s,
            this->z + s);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator+= (const T s)
    {
	    this->x += s;
	    this->y += s;
        this->z += s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator+ (const glm::_xvec3<T> & v) const
    {
	    return glm::_xvec3<T>(
		    this->x + v.x,
		    this->y + v.y,
            this->z + v.z);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator+= (const glm::_xvec3<T> & v)
    {
	    this->x += v.x;
	    this->y += v.y;
        this->z += v.z;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator- (const T s) const
    {
	    return glm::_xvec3<T>(
		    this->x - s,
		    this->y - s,
            this->z - s);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator-= (const T s)
    {
	    this->x -= s;
	    this->y -= s;
        this->z -= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator- (const glm::_xvec3<T> & v) const
    {
	    return _xvec3<T>(
		    this->x - v.x,
		    this->y - v.y,
            this->z - v.z);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator-= (const glm::_xvec3<T> & v)
    {
	    this->x -= v.x;
	    this->y -= v.y;
        this->z -= v.z;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator* (const T s) const
    {
	    return glm::_xvec3<T>(
		    this->x * s,
		    this->y * s,
            this->z * s);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator*= (const T s)
    {
	    this->x *= s;
	    this->y *= s;
        this->z *= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator* (const _xvec3<T> & v) const
    {
	    return _xvec3<T>(
		    this->x * v.x,
		    this->y * v.y,
            this->z * v.z);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator*= (const _xvec3<T> & v)
    {
	    this->x *= v.x;
	    this->y *= v.y;
        this->z *= v.z;
	    return *this;
    }

    template <typename T> 
    inline _xvec3<T> _xvec3<T>::operator* (const _xmat3<T>& m) const
    {
        return _xvec3<T>(
            m[0][0] * this->x + m[0][1] * this->y + m[0][2] * this->z,
            m[1][0] * this->x + m[1][1] * this->y + m[1][2] * this->z,
            m[2][0] * this->x + m[2][1] * this->y + m[2][2] * this->z);
    }

    template <typename T> 
    inline _xvec3<T>& _xvec3<T>::operator*= (const _xmat3<T>& m)
    {
        return *this = *this * m;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator/ (const T s) const
    {
	    return _xvec3<T>(
		    this->x / s,
		    this->y / s,
            this->z / s);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator/= (const T s)
    {
	    this->x /= s;
	    this->y /= s;
        this->z /= s;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator/ (const _xvec3<T> & v) const
    {
	    return _xvec3<T>(
		    this->x / v.x,
		    this->y / v.y,
            this->z / v.z);
    }

    template <typename T> 
    inline glm::_xvec3<T>& _xvec3<T>::operator/= (const _xvec3<T> & v)
    {
	    this->x /= v.x;
	    this->y /= v.y;
        this->z /= v.z;
	    return *this;
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator- () const
    {
	    return _xvec3<T>(
            -this->x, 
            -this->y,
            -this->z);
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator-- ()
    {
	    this->x--;
	    this->y--;
        this->z--;
	    return *this;
    }

    template <typename T> 
    inline const glm::_xvec3<T> _xvec3<T>::operator-- (int n) const 
    {
		return _xvec3<T>(
			this->x - T(1), 
			this->y - T(1), 
			this->z - T(1));
    }

    template <typename T> 
    inline glm::_xvec3<T> _xvec3<T>::operator++ ()
    {
	    this->x++;
	    this->y++;
        this->z++;
	    return *this;
    }

    template <typename T> 
    inline const glm::_xvec3<T> _xvec3<T>::operator++ (int n) const
    {
		return _xvec3<T>(
			this->x + T(1), 
			this->y + T(1), 
			this->z + T(1));
    }

    //////////////////////////////////////////////////////////////
    // bvec3 constructors

    inline _bvec3::_bvec3() :
        x(false),
        y(false),
        z(false)
    {}

    inline _bvec3::_bvec3(bool _x, bool _y, bool _z) :
        x(_x),
        y(_y),
        z(_z)
    {}

    inline _bvec3::_bvec3(bool b) :
        x(b),
        y(b),
        z(b)
    {}

    inline _bvec3::_bvec3(const _bvec2 & v1, bool v2) :
        x(v1.x),
        y(v1.y),
        z(v2)
    {}

    inline _bvec3::_bvec3(bool v1, const _bvec2 & v2) :
        x(v1),
        y(v2.x),
        z(v2.y)
    {}

    inline _bvec3::_bvec3(const _bvec4 & v) : 
        x(v.x),
        y(v.y),
        z(v.z)
    {}

    //////////////////////////////////////////////////////////////
    // bvec3 accesses

    inline bool& _bvec3::operator [] (int i)
    {
        return (&x)[i];
    }

    inline bool _bvec3::operator [] (int i) const
    {
        return (&x)[i];
    }

    inline _bvec3::operator bool* ()
    {
        return &x;
    }

    inline _bvec3::operator const bool* () const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // bvec2 conversions

    template <typename U>
    inline _bvec3::_bvec3(const _xvec3<U>& v) :
        x(bool(v.x)),
        y(bool(v.y)),
        z(bool(v.z))
    {}

    template <typename U> 
    inline _bvec3::_bvec3(const _xvec2<U>& v1, U v2) :
        x(bool(v1.x)),
        y(bool(v1.y)),
        z(bool(v2))
    {}

    template <typename U> 
    inline _bvec3::_bvec3(U v1, const _xvec2<U>& v2) :
        x(bool(v1)),
        y(bool(v2.x)),
        z(bool(v2.y))
    {}

    template <typename U> 
    inline _bvec3::_bvec3(const _xvec4<U>& v) :
        x(bool(v.x)),
        y(bool(v.y)),
        z(bool(v.z))
    {}

    //////////////////////////////////////////////////////////////
    // bvec3 operators

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    inline _bvec3& _bvec3::operator=(const _bvec3& x)
    {
        this->x = x.x;
        this->y = x.y;
        this->z = x.z;
        return *this;
    }

    inline _bvec3 _bvec3::operator--()
    {
	    this->x = !x;
	    this->y = !y;
        this->z = !z;
	    return *this;
    }

    inline _bvec3 _bvec3::operator++()
    {
	    this->x = !x;
	    this->y = !y;
        this->z = !z;
	    return *this;
    }

    inline const _bvec3 _bvec3::operator--(int n) const 
    {
		return _bvec3(
			!this->x, 
			!this->y, 
			!this->z);
    }

    inline const _bvec3 _bvec3::operator++(int n) const
    {
		return _bvec3(
			!this->x, 
			!this->y, 
			!this->z);
    }

    inline _bvec3 _bvec3::operator!() const 
    {
		return _bvec3(
			!this->x, 
			!this->y, 
			!this->z);
    }
} //namespace glm

#endif //__vec3_inl__


