//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-16
// Updated : 2006-01-17
// Licence : This source is under GNU LGPL licence
// File    : _vec4.inl
//////////////////////////////////////////////////////////////////////////////////

#ifndef __vec4_inl__
#define __vec4_inl__

#include "./_vec2.h"
#include "./_vec3.h"
#include "./_vec4.h"

namespace glm
{
    //////////////////////////////////////////////////////////////
    // vec4 and ivec4 constructors

    template <typename T> 
    inline _xvec4<T>::_xvec4() :
        x(0),
        y(0),
        z(0),
        w(0)
    {}

    //data(_mm_load1_ps(&f))
    template <typename T> 
    inline _xvec4<T>::_xvec4(T f) :
        x(f),
        y(f),
        z(f),
        w(f)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(const T _x, const T _y, const T _z, const T _w) :
        x(_x),
        y(_y),
        z(_z),
        w(_w)
    {}
/*
    template <typename T> 
    inline _xvec4<T>::_xvec4(const T* a) :
        x(a[0]),
        y(a[1]),
        z(a[2]),
        w(a[3])
    {}
*/
    //////////////////////////////////////////////////////////////
    // vec4 and ivec4 conversions

    template <typename T> 
    inline _xvec4<T>::_xvec4(const _xvec2<T> & v1, const _xvec2<T> & v2) :
        x(v1.x),
        y(v1.y),
        z(v2.x),
        w(v2.y)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(const _xvec3<T> & v1, T v2) :
        x(v1.x),
        y(v1.y),
        z(v1.z),
        w(v2)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(T v1, const _xvec3<T> & v2) :
        x(v1),
        y(v2.x),
        z(v2.y),
        w(v2.z)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(T v1, T v2, const _xvec2<T> & v3) :
        x(v1),
        y(v2),
        z(v3.x),
        w(v3.y)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(T v1, const _xvec2<T> & v2, T v3) :
        x(v1),
        y(v2.x),
        z(v2.y),
        w(v3)
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(const _xvec2<T> & v1, T v2, T v3) :
        x(v1.x),
        y(v1.y),
        z(v2),
        w(v3)
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(U x, U y, U z, U w) :
        x(T(x)),
        y(T(y)),
        z(T(z)),
        w(T(w))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(const _xvec4<U>& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z)),
        w(T(v.w))
    {}

    template <typename T>
    template <typename U>
    inline _xvec4<T>::_xvec4(const _xvec2<U>& v1, const _xvec2<U>& v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2.x)),
        w(T(v2.y))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(const _xvec3<U>& v1, U v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v1.z)),
        w(T(v2))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(U v1, const _xvec3<U>& v2) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y)),
        w(T(v2.z))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(U v1, U v2, const _xvec2<U>& v3) :
        x(T(v1)),
        y(T(v2)),
        z(T(v3.x)),
        w(T(v3.y))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(U v1, const _xvec2<U>& v2, U v3) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y)),
        w(T(v3))
    {}

    template <typename T>
    template <typename U> 
    inline _xvec4<T>::_xvec4(const _xvec2<U>& v1, U v2, U v3) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2)),
        w(T(v3))
    {}

    template <typename T> 
    inline _xvec4<T>::_xvec4(const glm::_bvec4& v) :
        x(T(v.x)),
        y(T(v.y)),
        z(T(v.z)),
        w(T(v.w))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(const _bvec2& v1, const _bvec2& v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2.x)),
        w(T(v2.y))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(const _bvec3& v1, bool v2) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v1.z)),
        w(T(v2))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(bool v1, const _bvec3& v2) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y)),
        w(T(v2.z))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(bool v1, bool v2, const _bvec2& v3) :
        x(T(v1)),
        y(T(v2)),
        z(T(v3.x)),
        w(T(v3.y))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(bool v1, const _bvec2& v2, bool v3) :
        x(T(v1)),
        y(T(v2.x)),
        z(T(v2.y)),
        w(T(v3))
    {}

    template <typename T>
    inline _xvec4<T>::_xvec4(const _bvec2& v1, bool v2, bool v3) :
        x(T(v1.x)),
        y(T(v1.y)),
        z(T(v2)),
        w(T(v3))
    {}

    //////////////////////////////////////////////////////////////
    // vec4 and ivec4 accesses

    template <typename T> 
    inline T& _xvec4<T>::operator[](int i)
    {
        return (&x)[i];
    }

    template <typename T> 
    inline T _xvec4<T>::operator[](int i) const
    {
        return (&x)[i];
    }

    template <typename T> 
    inline _xvec4<T>::operator T*()
    {
        return &x;
    }

    template <typename T> 
    inline _xvec4<T>::operator const T*() const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // vec4 and ivec4 operators definitions

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator=(const glm::_xvec4<T>& x)
    {
        this->x = x.x;
        this->y = x.y;
        this->z = x.z;
        this->w = x.w;
        return *this;
    }

    //_mm_add_ps
    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator+ (const T s) const
    {
	    return _xvec4<T>(
		    this->x + s,
		    this->y + s,
            this->z + s,
            this->w + s);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator+= (const T s)
    {
	    this->x += s;
	    this->y += s;
        this->z += s;
        this->w += s;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator+(const glm::_xvec4<T> & v) const
    {
	    return _xvec4<T>(
		    this->x + v.x,
		    this->y + v.y,
            this->z + v.z,
            this->w + v.w);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator+=(const glm::_xvec4<T> & v)
    {
	    this->x += v.x;
	    this->y += v.y;
        this->z += v.z;
        this->w += v.w;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator- (const T s) const
    {
	    return _xvec4<T>(
		    this->x - s,
		    this->y - s,
            this->z - s,
            this->w - s);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator-= (const T s)
    {
	    this->x -= s;
	    this->y -= s;
        this->z -= s;
        this->w -= s;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator-(const glm::_xvec4<T> & v) const
    {
	    return _xvec4<T> (
		    this->x - v.x,
		    this->y - v.y,
            this->z - v.z,
            this->w - v.w);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator-=(const glm::_xvec4<T> & v)
    {
	    this->x -= v.x;
	    this->y -= v.y;
        this->z -= v.z;
        this->w -= v.w;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator*(const T s) const
    {
	    return _xvec4<T>(
		    this->x * s,
		    this->y * s,
            this->z * s,
            this->w * s);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator*=(const T s)
    {
	    this->x *= s;
	    this->y *= s;
        this->z *= s;
        this->w *= s;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator* (const _xvec4<T> & v) const
    {
	    return _xvec4<T>(
		    this->x * v.x,
		    this->y * v.y,
            this->z * v.z,
            this->w * v.w);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator*= (const _xvec4<T> & v)
    {
	    this->x *= v.x;
	    this->y *= v.y;
        this->z *= v.z;
        this->w *= v.w;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator* (const _xmat4<T>& m) const
    {
        return _xvec4<T>(
            m[0][0] * this->x + m[0][1] * this->y + m[0][2] * this->z + m[0][3] * this->w,
            m[1][0] * this->x + m[1][1] * this->y + m[1][2] * this->z + m[1][3] * this->w,
            m[2][0] * this->x + m[2][1] * this->y + m[2][2] * this->z + m[2][3] * this->w,
            m[3][0] * this->x + m[3][1] * this->y + m[3][2] * this->z + m[3][3] * this->w);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator*= (const _xmat4<T>& m)
    {
        return *this = *this * m;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator/(const T s) const
    {
	    return _xvec4<T>(
		    this->x / s,
		    this->y / s,
            this->z / s,
            this->w / s);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator/=(const T s)
    {
	    this->x /= s;
	    this->y /= s;
        this->z /= s;
        this->w /= s;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator/ (const _xvec4<T> & v) const
    {
	    return _xvec4<T>(
		    this->x / v.x,
		    this->y / v.y,
            this->z / v.z,
            this->w / v.w);
    }

    template <typename T> 
    inline _xvec4<T>& _xvec4<T>::operator/= (const _xvec4<T> & v)
    {
	    this->x /= v.x;
	    this->y /= v.y;
        this->z /= v.z;
        this->w /= v.w;
	    return *this;
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator-() const
    {
	    return _xvec4<T>(
			-this->x, 
			-this->y,
            -this->z,
            -this->w);
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator--()
    {
	    this->x--;
	    this->y--;
        this->z--;
        this->w--;
	    return *this;
    }

    template <typename T> 
    inline const _xvec4<T> _xvec4<T>::operator--(int n) const 
    {
		return _xvec4<T>(
			this->x - T(1), 
			this->y - T(1), 
			this->z - T(1), 
			this->w - T(1));
    }

    template <typename T> 
    inline _xvec4<T> _xvec4<T>::operator++()
    {
	    this->x++;
	    this->y++;
        this->z++;
        this->w++;
	    return *this;
    }

    template <typename T> 
    inline const _xvec4<T> _xvec4<T>::operator++(int n) const
    {
		return _xvec4<T>(
			this->x + T(1), 
			this->y + T(1), 
			this->z + T(1), 
			this->w + T(1));
    }

    //////////////////////////////////////////////////////////////
    // vec4 and ivec4 privates functions

    //////////////////////////////////////////////////////////////
    // bvec4 constructors

    inline _bvec4::_bvec4() :
        x(false),
        y(false),
        z(false),
        w(false)
    {}

    inline _bvec4::_bvec4(bool x, bool y, bool z, bool w) :
        x(x),
        y(y),
        z(z),
        w(w)
    {}

    inline _bvec4::_bvec4(const _bvec2& v1, const _bvec2& v2) :
        x(v1.x),
        y(v1.y),
        z(v2.x),
        w(v2.y)
    {}

    inline _bvec4::_bvec4(const _bvec3& v1, bool v2) :
        x(v1.x),
        y(v1.y),
        z(v1.z),
        w(v2)
    {}

    inline _bvec4::_bvec4(bool v1, const _bvec3 & v2) :
        x(v1),
        y(v2.x),
        z(v2.y),
        w(v2.z)
    {}

    inline _bvec4::_bvec4(bool v1, bool v2, const _bvec2& v3) :
        x(v1),
        y(v2),
        z(v3.x),
        w(v3.y)
    {}

    inline _bvec4::_bvec4(bool v1, const _bvec2& v2, bool v3) :
        x(v1),
        y(v2.x),
        z(v2.y),
        w(v3)
    {}

    inline _bvec4::_bvec4(const _bvec2& v1, bool v2, bool v3) :
        x(v1.x),
        y(v1.y),
        z(v2),
        w(v3)
    {}

    //////////////////////////////////////////////////////////////
    // bvec4 conversions

    template <typename U>
    inline _bvec4::_bvec4(const _xvec4<U>& v) :
        x(bool(v.x)),
        y(bool(v.y)),
        z(bool(v.z)),
        w(bool(v.w))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(U x, U y, U z, U w) :
        x(bool(x)),
        y(bool(y)),
        z(bool(z)),
        w(bool(w))
    {}

    template <typename U>
    inline _bvec4::_bvec4(const _xvec2<U>& v1, const _xvec2<U>& v2) :
        x(bool(v1.x)),
        y(bool(v1.y)),
        z(bool(v2.x)),
        w(bool(v2.y))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(const _xvec3<U>& v1, U v2) :
        x(bool(v1.x)),
        y(bool(v1.y)),
        z(bool(v1.z)),
        w(bool(v2))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(U v1, const _xvec3<U>& v2) :
        x(bool(v1)),
        y(bool(v2.x)),
        z(bool(v2.y)),
        w(bool(v2.z))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(U v1, U v2, const _xvec2<U>& v3) :
        x(bool(v1)),
        y(bool(v2)),
        z(bool(v3.x)),
        w(bool(v3.y))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(U v1, const _xvec2<U>& v2, U v3) :
        x(bool(v1)),
        y(bool(v2.x)),
        z(bool(v2.y)),
        w(bool(v3))
    {}

    template <typename U> 
    inline _bvec4::_bvec4(const _xvec2<U>& v1, U v2, U v3) :
        x(bool(v1.x)),
        y(bool(v1.y)),
        z(bool(v2)),
        w(bool(v3))
    {}

    //////////////////////////////////////////////////////////////
    // bvec4 accesses

    inline bool& _bvec4::operator[](int i)
    {
        return (&x)[i];
    }

    inline bool _bvec4::operator[](int i) const
    {
        return (&x)[i];
    }

    inline _bvec4::operator bool*()
    {
        return &x;
    }

    inline _bvec4::operator const bool*() const 
    {
        return &x;
    }

    //////////////////////////////////////////////////////////////
    // bvec4 operators

    // This function shouldn't required but it seems that VC7.1 have an optimisation bug if this operator wasn't declared
    inline _bvec4& _bvec4::operator=(const _bvec4& x)
    {
        this->x = x.x;
        this->y = x.y;
        this->z = x.z;
        this->w = x.w;
        return *this;
    }

    inline _bvec4 _bvec4::operator--()
    {
	    this->x = !x;
	    this->y = !y;
        this->z = !z;
        this->w = !w;
	    return *this;
    }

    inline _bvec4 _bvec4::operator++()
    {
	    this->x = !x;
	    this->y = !y;
        this->z = !z;
        this->w = !w;
	    return *this;
    }

    inline const _bvec4 _bvec4::operator--(int n) const 
    {
		return _bvec4(
			!this->x, 
			!this->y, 
			!this->z, 
			!this->w);
    }

    inline const _bvec4 _bvec4::operator++(int n) const
    {
		return _bvec4(
			!this->x, 
			!this->y, 
			!this->z, 
			!this->w);
    }

    inline _bvec4 _bvec4::operator!() const 
    {
		return _bvec4(
			!this->x, 
			!this->y, 
			!this->z, 
			!this->w);
    }
} //namespace glm

#endif //__vec4_inl__
