namespace o {
namespace core {

	// CCreator
	template <typename base, typename derived>
	CCreator<base, derived>::CCreator()
	{}

	template <typename base, typename derived>
	CCreator<base, derived>::~CCreator()
	{}

	template <typename base, typename derived>
	base* CCreator<base, derived>::Create()
	{
		return new_object(derived);
	}

	template <typename base, typename derived>
	bool CCreator<base, derived>::Destroy(base* pObject)
	{
		if(!pObject)
			return false;
		delete_object(pObject);
		return true;
	}

	// CAbstractFactory
	template<typename key, typename base>
	CAbstractFactory<key, base>::CAbstractFactory()
	{}

	template<typename key, typename base>
	CAbstractFactory<key, base>::~CAbstractFactory()
	{
		for
		(
			TCreatorsIt it = CreatorsMap.begin();
			it != CreatorsMap.end();
			it++
		)
		{
			delete_object(it->second);
		}
	}

	template<typename key, typename base>
	bool CAbstractFactory<key, base>::Register(key Key, ICreator<base>* pCreator)
	{
		if(CreatorsMap.find(Key) != CreatorsMap.end())
			return false;
		CreatorsMap.insert(std::make_pair(Key, pCreator));
		return true;
	}

	template<typename key, typename base>
	bool CAbstractFactory<key, base>::UnRegister(key Key)
	{
		TCreatorsIt it = CreatorsMap.find(Key);
		if(it == CreatorsMap.end())
			return false;
		delete_object(it->second);
		CreatorsMap.erase(it);
		return true;
	}

	template<typename key, typename base>
	base* CAbstractFactory<key, base>::CreateObject(key Key) const
	{
		TCreatorsCit it = CreatorsMap.find(Key);
		if(it == CreatorsMap.end())
			return 0;
		return it->second->Create();
	}

	template<typename key, typename base>
	bool CAbstractFactory<key, base>::DestroyObject(base* pObject)
	{
		if(!pObject)
			return false;
		delete_object(pObject);
		return true;
	}

	// creator
	template <typename base, typename derived>
	CCreator<base, derived>* creator()
	{
		typedef typename CCreator<base, derived> TCreator;

		return new_object(TCreator);
		//return new_object(CCreator<base, derived>);
		//return new CCreator<base, derived>;
	}

}//namespace core
}//namespace o
